DBQuest = LibStub("AceAddon-3.0"):NewAddon("DBQuest", "AceEvent-3.0")
local DBQuest = DBQuest
DBQuest.title = "DBQuest"
DBQuest.version = "r5"

local L = LibStub("AceLocale-3.0"):GetLocale("DBQuest")

DBQuest.defaults = {
	char = {
		enableAccept = false,
		enableTurnin = false,
		enableShare = false,
	}
}

DBQuest.options = {
	name = "DBQuest",
	desc = L["Dual box quest helper"],
	type = 'group',
	args = {
		accept = {
			type = 'toggle',
			name = L["Accept"],
			desc = L["Accept quests"],
			get = function(info)
				return DBQuest.db.char.enableAccept
			end,
			set = function(info, value)
				DBQuest.db.char.enableAccept = value
			end,
		},
		turnin = {
			type = 'toggle',
			name = L["Turn In"],
			desc = L["Turn in quests with no reward choices"],
			get = function(info)
				return DBQuest.db.char.enableTurnin
			end,
			set = function(info, value)
				DBQuest.db.char.enableTurnin = value
			end,
		},
		share = {
			type = 'toggle',
			name = L["Share"],
			desc = L["Share new quests"],
			get = function(info)
				return DBQuest.db.char.enableShare
			end,
			set = function(info, value)
				DBQuest.db.char.enableShare = value
			end,
		},
	},
}

local qcache = {}

local function updateqcache()
	local i = 0
	qcache = {}
	while (GetQuestLogTitle(i+1) ~= nil) do
		i = i + 1
		local title, _, _, _, header = GetQuestLogTitle(i)
		if not header then qcache[title] = 1 end
	end
end

function DBQuest:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("DBQuestDB", self.defaults, "Default")

	LibStub("AceConfig-3.0"):RegisterOptionsTable("DBQuest", self.options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DBQuest", "DBQuest")

	local opencfg = function()
		LibStub("AceConfigDialog-3.0"):Open("DBQuest")
	end
	LibStub("AceConsole-3.0"):RegisterChatCommand("dbquest", opencfg)
	LibStub("AceConsole-3.0"):RegisterChatCommand("dbq", opencfg)

	self:RegisterEvent("QUEST_COMPLETE")
	self:RegisterEvent("QUEST_DETAIL")
	self:RegisterEvent("QUEST_GREETING")
	self:RegisterEvent("QUEST_ACCEPT_CONFIRM")
	self:RegisterEvent("QUEST_PROGRESS")
	self:RegisterEvent("QUEST_LOG_UPDATE")
	self:RegisterEvent("GOSSIP_SHOW")
	updateqcache()
end

function DBQuest:QUEST_ACCEPT_CONFIRM(event)
	if self.db.char.enableAccept then
		ConfirmAcceptQuest()
	end
end

function DBQuest:QUEST_DETAIL(event)
	if self.db.char.enableAccept then
		AcceptQuest()
	end
end

function DBQuest:QUEST_PROGRESS(event)
	if self.db.char.enableTurnin and IsQuestCompletable() then
		CompleteQuest()
	end
end

function DBQuest:GOSSIP_SHOW(event)
	if self.db.char.enableTurnin then
		local quests = { GetGossipActiveQuests() }
		for i = 1, #quests, 4 do
			if (quests[i+3]) then
				SelectGossipActiveQuest(i)
				return
			end
		end
	end

	if self.db.char.enableAccept and GetGossipAvailableQuests() then
		SelectGossipAvailableQuest(1)
	end
end

function DBQuest:QUEST_GREETING(event)
	if self.db.char.enableTurnin then
		for i = 1, GetNumActiveQuests() do
			local _, complete = GetActiveTitle(i)
			if complete then
				SelectActiveQuest(i)
				return
			end
		end
	end

	if self.db.char.enableAccept and GetNumAvailableQuests() > 0 then
		SelectAvailableQuest(1)
	end
end

function DBQuest:QUEST_COMPLETE(event)
	if self.db.char.enableTurnin and GetNumQuestChoices() < 2 then
		GetQuestReward(1)
	end
end

function DBQuest:PLAYER_LOGIN(event)
	updateqcache()
end

local function sharenew()
	local i = 0
	local changed
	local oldq = GetQuestLogSelection()

	while (GetQuestLogTitle(i+1) ~= nil) do
		i = i + 1
		local title, _, _, _, header = GetQuestLogTitle(i)
		if not header then
			changed = true
			SelectQuestLogEntry(i)
			if not qcache[title] and GetQuestLogPushable() then
				QuestLogPushQuest()
				SelectQuestLogEntry(oldq)
				return
			end
		end
	end

	if changed then
		SelectQuestLogEntry(oldq)
	end
end

function DBQuest:QUEST_LOG_UPDATE(event)
	if self.db.char.enableShare then
		sharenew()
	end
	updateqcache()
end
